﻿#Region " Imports "
Imports Infragistics.Web.UI.GridControls
Imports Data.Tsql

#End Region
<Serializable()> _
Public Class GridColumnCollection
    Inherits System.Collections.CollectionBase

#Region " Constants "
    Private Const HIDDEN_FIELDNAME_PREFIX As String = "Hidden "
#End Region

#Region "Constructors"
    Public Sub New()
        MyBase.New()
    End Sub

    Public Sub New(ByVal gridColumns As Infragistics.Web.UI.GridControls.GridFieldCollection)
        MyBase.New()
        Me.Parse(gridColumns)
    End Sub

    Public Sub New(ByVal gridColumns As Infragistics.Web.UI.GridControls.GridFieldCollection, ByVal bExcludeHiddenColumns As Boolean)
        MyBase.New()
        Me.Parse(gridColumns, bExcludeHiddenColumns)
    End Sub

    Public Sub New(ByVal strFields As String)
        MyBase.New()
        Me.Parse(strFields)
    End Sub

    Public Sub New(ByVal strFields As String, ByVal bHiddenFields As Boolean)
        MyBase.New()
        Me.Parse(strFields, bHiddenFields)
    End Sub

    Public Sub New(ByVal gridColumns As GridGroupedColumnCollection)
        MyBase.New()
        Me.Parse(gridColumns)
    End Sub
#End Region

#Region "Properties"
    Public ReadOnly Property Item(ByVal nIndex As Integer) As GridColumn
        Get
            Return CType(Me.List.Item(nIndex), GridColumn)
        End Get
    End Property

    'Public ReadOnly Property Items() As GridColumnCollection
    '    Get
    '        Return Me
    '    End Get
    'End Property
#End Region

#Region "Methods"
    Public Sub Add(ByVal gcColumn As GridColumn)
        Me.List.Add(gcColumn)
    End Sub

    Public Function Contains(ByVal gcColumn As GridColumn) As Boolean
        Return Me.List.ToString.Contains(gcColumn.Column)
    End Function

    Public Sub CopyTo(ByVal arrGroupBy() As GridColumn, ByVal nIndex As Integer)
        Me.CopyTo(arrGroupBy, nIndex)
    End Sub

    Public Sub Insert(ByVal nIndex As Integer, ByVal gcColumn As GridColumn)
        Me.List.Insert(nIndex, gcColumn)
    End Sub

    Public Function IndexOf(ByVal scGroupBy As GridColumn) As Integer
        Return Me.List.IndexOf(scGroupBy)
    End Function

    Public Overloads Sub Parse(ByVal GridColumns As Infragistics.Web.UI.GridControls.GridFieldCollection)
        If (GridColumns Is Nothing) Then
            Exit Sub
        End If

        Me.Clear()

        If (GridColumns.Count = 0) Then
            Exit Sub
        End If
        For Each grdcolumn As BoundDataField In GridColumns
            Dim col As New GridColumn(grdcolumn)
            Me.Add(col)
        Next
    End Sub

    Public Overloads Sub Parse(ByVal GridColumns As Infragistics.Web.UI.GridControls.GridFieldCollection, _
                               ByVal bExcludeHiddenColumns As Boolean)
        If (GridColumns Is Nothing) Then
            Exit Sub
        End If

        Me.Clear()

        If (GridColumns.Count = 0) Then
            Exit Sub
        End If

        Dim i As Integer
        For i = 0 To GridColumns.Count - 1
            If (bExcludeHiddenColumns) Then ' Sanitize GridField Collection
                If GridColumns.Item(i).Hidden Then
                    ' Ignore
                ElseIf (GridColumns.Item(i).Type) = GetType(TemplateDataField) Then
                    ' Ignore
                Else
                    Dim col As New GridColumn(GridColumns.Item(i))
                    Me.Add(col)
                End If
            End If

        Next
    End Sub

    Public Overloads Sub Parse(ByVal strFields As String)
        If (strFields Is Nothing) Then
            Exit Sub
        End If

        Me.Clear()

        If (strFields.Length = 0) Then
            Exit Sub
        End If

        Dim strParts() As String = strFields.Split(Keyword.FieldDelimiter)
        Dim nPart As Integer
        For nPart = 0 To strParts.Length - 1
            Dim grdcolumn As New GridColumn(strParts(nPart).Trim())
            Me.Add(grdcolumn)
        Next
    End Sub

    Public Overloads Sub Parse(ByVal strFields As String, ByVal bHiddenFields As Boolean)
        If (strFields Is Nothing) Then
            Exit Sub
        End If

        Me.Clear()

        If (strFields.Length = 0) Then
            Exit Sub
        End If

        Dim strParts() As String = strFields.Split(Keyword.FieldDelimiter)
        Dim nPart As Integer
        For nPart = 0 To strParts.Length - 1
            If bHiddenFields Then
                Dim grdcolumn As New GridColumn(String.Concat(HIDDEN_FIELDNAME_PREFIX, strParts(nPart).Trim()))
                grdcolumn.Hidden = bHiddenFields
                Me.Add(grdcolumn)
            Else
                Dim grdcolumn As New GridColumn(strParts(nPart).Trim())
                grdcolumn.Hidden = bHiddenFields
                Me.Add(grdcolumn)
            End If
        Next
    End Sub

    Public Overloads Sub Parse(ByVal GridColumns As GridGroupedColumnCollection)
        If (GridColumns Is Nothing) Then
            Exit Sub
        End If

        Me.Clear()

        If (GridColumns.Count = 0) Then
            Exit Sub
        End If
        For Each grdcolumn As GridGroupedColumn In GridColumns
            Dim col As New GridColumn(grdcolumn)
            Me.Add(col)
        Next
    End Sub

    Public Sub Remove(ByVal nIndex As Integer)
        Me.List.RemoveAt(nIndex)
    End Sub

    Public Shadows Function Count() As Integer
        Return Me.List.Count()
    End Function
#End Region

#Region " Overrides "
    ''' <summary>
    ''' Gets a <c>String</c> representation of a <c>GridColumnCollection</c>.
    ''' </summary>
    ''' <returns>A <c>String</c> representation of the <c>GridColumnCollection</c>.</returns>
    ''' <remarks>This method returns a human-readable string.
    ''' </remarks>
    Public Overloads Overrides Function ToString() As String
        Return ToString(","c)
    End Function

    ''' <summary>
    ''' Gets a <c>String</c> representation of a <c>GridColumnCollection</c>.
    ''' </summary>
    ''' <param name="chrDelimiter">A <c>Char</c> used to separate field names.
    ''' </param>
    ''' <returns>A <c>String</c> representation of the <c>GridColumnCollection</c> in 
    ''' which field names are delimited by <paramref name="chrDelimiter"/>.</returns>
    ''' <remarks>This method returns a human-readable string.
    ''' </remarks>
    Public Overloads Function ToString(ByVal chrDelimiter As Char) As String

        Dim strOutput As String = ""
        Dim nField As Integer

        For nField = 0 To Me.Count - 1
            If (nField > 0) Then
                strOutput = String.Concat(strOutput, chrDelimiter)
            End If
            strOutput = String.Concat(strOutput, Me.Item(nField).Column)
        Next

        Return strOutput
    End Function

    Public Overloads Function ToSql() As String
        Return Me.ToSql(","c, """"c, """"c)
    End Function

    Public Overloads Function ToSql(ByVal strPrefix As String) As String
        Return Me.ToSql(strPrefix, ","c, """"c, """"c)
    End Function

    ''' <summary>
    ''' Gets a <c>String</c> representation of a <c>GridColumnCollection</c>.
    ''' </summary>
    ''' <param name="chrDelimiter">A <c>Char</c> used to separate field names.
    ''' </param>
    ''' <param name="chrOpenQuote">A <c>Char</c> that is placed around field names.
    ''' </param>
    ''' <param name="chrCloseQuote">A <c>Char</c> that is placed around field names.
    ''' </param>
    ''' <returns>A <c>String</c> representation of the <c>GridColumnCollection</c> in 
    ''' which field names are delimited by <paramref name="chrDelimiter" /> and
    ''' enclosed in <paramref name="chrQuote" /> characters.</returns>
    ''' <remarks>This method returns a human-readable string.
    ''' </remarks>
    Public Overloads Function ToSql(ByVal chrDelimiter As Char, ByVal chrOpenQuote As Char, ByVal chrCloseQuote As Char) As String
        Dim strOutput As String = ""
        Dim nField As Integer

        For nField = 0 To Me.Count - 1
            If (nField > 0) Then
                strOutput = String.Concat(strOutput, chrDelimiter)
            End If
            strOutput = String.Concat(strOutput, chrOpenQuote, Me.Item(nField).Column, chrCloseQuote)
        Next
        Return strOutput
    End Function

    ''' <summary>
    ''' Gets a <c>String</c> representation of a <c>GridColumnCollection</c>.
    ''' </summary>
    ''' <param name="strPrefix">A <c>String</c> to prefix field names with.</param>
    ''' <param name="chrDelimiter">A <c>Char</c> used to separate field names.
    ''' </param>
    ''' <param name="chrOpenQuote">A <c>Char</c> that is placed around field names.
    ''' </param>
    ''' <param name="chrCloseQuote">A <c>Char</c> that is placed around field names.
    ''' </param>
    ''' <returns>A <c>String</c> representation of the <c>GridColumnCollection</c> in 
    ''' which field names are delimited by <paramref name="chrDelimiter" />,
    ''' prefixed with <paramref name="strPrefix"/> and enclosed in <paramref 
    ''' name="chrQuote" /> characters.</returns>
    ''' <remarks>This method returns a human-readable string.
    ''' </remarks>
    Public Overloads Function ToSql(ByVal strPrefix As String, ByVal chrDelimiter As Char, ByVal chrOpenQuote As Char, ByVal chrCloseQuote As Char) As String
        Dim strOutput As String = ""
        Dim nField As Integer

        For nField = 0 To Me.Count - 1
            If (nField > 0) Then
                strOutput = String.Concat(strOutput, chrDelimiter)
            End If
            strOutput = String.Concat(strOutput, chrOpenQuote, strPrefix, Me.Item(nField).Column, chrCloseQuote)
        Next
        Return strOutput
    End Function

    ''' <summary>
    ''' Gets an SQL alias <c>String</c> for a <c>GridColumnCollection</c>.
    ''' </summary>
    ''' <param name="strAliasPrefix">A <c>String</c> to prefix field names with.
    ''' </param>
    ''' <returns>An SQL alias <c>String</c> that can be used to select fields from 
    ''' a table and return a resultset in which field names are prefixed with 
    ''' <paramref name="strAliasPrefix" />.</returns>
    Public Overloads Function ToSqlAliasString(ByVal strAliasPrefix As String) As String
        Return ToSqlAliasString(Keyword.FieldDelimiter, Keyword.OpenQuote, Keyword.CloseQuote, strAliasPrefix)
    End Function

    ''' <summary>
    ''' Gets an SQL alias <c>String</c> for a <c>GridColumnCollection</c>.
    ''' </summary>
    ''' <param name="chrDelimiter">A <c>Char</c> used to separate field names.
    ''' </param>
    ''' <param name="chrOpenQuote">A <c>Char</c> that is placed around field names.
    ''' </param>
    ''' <param name="chrCloseQuote">A <c>Char</c> that is placed around field names.
    ''' </param>
    ''' <param name="strAliasPrefix">A <c>String</c> to prefix field names with.
    ''' </param>
    ''' <returns>An SQL alias <c>String</c> that can be used to select fields from 
    ''' a table and return a resultset in which field names are prefixed with 
    ''' <paramref name="strAliasPrefix" />.</returns>
    Public Overloads Function ToSqlAliasString(ByVal chrDelimiter As Char, ByVal chrOpenQuote As Char, ByVal chrCloseQuote As Char, ByVal strAliasPrefix As String) As String
        Dim strOutput As String = ""
        Dim nField As Integer

        For nField = 0 To Me.Count - 1
            If (nField > 0) Then
                strOutput = String.Concat(strOutput, chrDelimiter)
            End If

            Dim strField As String = String.Format("{0}{2}{1}{3}{0}{4}{2}{1}", chrOpenQuote, chrCloseQuote, Item(nField).Column, Keyword.As, strAliasPrefix)
            strOutput = String.Concat(strOutput, strField)
        Next
        Return strOutput
    End Function
#End Region
End Class
