﻿#Region " Imports"
Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Web
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports System.ComponentModel

Imports Infragistics.Documents
Imports Infragistics.Web.UI.GridControls

Imports Data
Imports Grid

#End Region

Public Class GridPagerUserControl
    Inherits System.Web.UI.UserControl

#Region " Constants"
    'Defaults
    Private Const DEFAULT_ID As String = "PAGER_ID"
    Private Const DEFAULT_VIEW_NAME As String = "Default List"
    Private Const DEFAULT_GRID_ROWS As Integer = 30
    Private Const DEFAULT_XLS_FILENAME As String = "Export.xls"
    Private Const DEFAULT_WORKSHEET_NAME As String = "Export"
    Private Const EXCEL_MAX_FIELD_LENGTH As Integer = 32767
    Private Const EXCEL_MAX_FIELD_LENGTH_ERROR As String = "The grid data cannot be exported to excel. A field or fields exceed the maximum allowed Excel cell field length."
    Private Const IMAGE_DIRECTORY As String = "~/images/webgrid/pager/"
#End Region

#Region " Declarations "
    Private _id As String
    Private _totalRecordCount As Integer
    Private _pageSize As Integer
    Private _pageNumber As Integer
    Private _excelfileName As String
    Private _worksheetName As String
    Private _excelDataExportMode As DataExportMode
    Private _gridNumberOfRows As Integer
    Private _gridType As GridType
    Private _selectedViewIndex As Integer
    Private _selectedViewName As String
    Private _showViews As Boolean
    Private _showPager As Boolean
    Private _showExcelExport As Boolean
#End Region

#Region " Event Handlers "
    Delegate Sub PageChangedEventHandler(ByVal sender As Object, ByVal e As GridPagerSettingsChangedEventArgs)
    Public Event PageChanged As PageChangedEventHandler

    Delegate Sub ExportedToExcelEventHandler(ByVal sender As Object, ByVal e As GridPagerExportedToExcelEventArgs)
    Public Event ExportedToExcel As ExportedToExcelEventHandler

    Delegate Sub NumberOfRowsChangedEventHandler(ByVal sender As Object, ByVal e As GridPagerPageSizeChangedEventArgs)
    Public Event NumberOfRowsChanged As NumberOfRowsChangedEventHandler

    Delegate Sub ViewChangedEventHandler(ByVal sender As Object, ByVal e As GridPagerViewChangedEventArgs)
    Public Event ViewChanged As ViewChangedEventHandler
#End Region

#Region " Constructors "
    Public Sub New()

    End Sub
#End Region

#Region " Properties "
    Protected Property TotalRecordCount As Integer
        Get
            If Me.ViewState(ViewStateVariable.PagerTotalRecordCount) IsNot Nothing Then
                Return Convert.ToInt32(Me.ViewState(ViewStateVariable.PagerTotalRecordCount))
            Else
                Me.ViewState.Add(ViewStateVariable.PagerTotalRecordCount, 0)
                Return 0
            End If
        End Get
        Set(value As Integer)
            If Me.ViewState(ViewStateVariable.PagerTotalRecordCount) IsNot Nothing Then
                Me.ViewState(ViewStateVariable.PagerTotalRecordCount) = value
            Else
                Me.ViewState.Add(ViewStateVariable.PagerTotalRecordCount, value)
            End If
        End Set
    End Property

    Protected Property PageSize As Integer
        Get
            If Me.ViewState(ViewStateVariable.PagerPageSize) IsNot Nothing Then
                Return Convert.ToInt32(Me.ViewState(ViewStateVariable.PagerPageSize))
            Else
                Me.ViewState.Add(ViewStateVariable.PagerPageSize, DEFAULT_GRID_ROWS)
                Return DEFAULT_GRID_ROWS
            End If
        End Get
        Set(value As Integer)
            If Me.ViewState(ViewStateVariable.PagerPageSize) IsNot Nothing Then
                Me.ViewState(ViewStateVariable.PagerPageSize) = value
            Else
                Me.ViewState.Add(ViewStateVariable.PagerPageSize, value)
            End If
        End Set
    End Property

    Protected ReadOnly Property TotalNumberOfPages() As Integer
        Get
            Return CInt(Math.Truncate(Math.Ceiling(CType(Me.TotalRecordCount, Double) / CType(Me.PageSize, Double))))
        End Get
    End Property

    Protected Property PageNumber As Integer
        Get
            If Me.ViewState(ViewStateVariable.GridPageNumber) IsNot Nothing Then
                Return Convert.ToInt32(Me.ViewState(ViewStateVariable.GridPageNumber))
            Else
                Me.ViewState.Add(ViewStateVariable.GridPageNumber, 1)
                Return 1
            End If
        End Get
        Set(value As Integer)
            If Me.ViewState(ViewStateVariable.GridPageNumber) IsNot Nothing Then
                Me.ViewState(ViewStateVariable.GridPageNumber) = value
            Else
                Me.ViewState.Add(ViewStateVariable.GridPageNumber, value)
            End If
        End Set
    End Property

    Protected Property NumberOfRows As Integer
        Get
            If Me.ViewState(ViewStateVariable.PagerGridRecords) IsNot Nothing Then
                Return Convert.ToInt32(Me.ViewState(ViewStateVariable.PagerGridRecords))
            Else
                Me.ViewState.Add(ViewStateVariable.PagerGridRecords, GridPagerUserControl.DEFAULT_GRID_ROWS)
                Return GridPagerUserControl.DEFAULT_GRID_ROWS
            End If
        End Get
        Set(value As Integer)
            If Me.ViewState(ViewStateVariable.PagerGridRecords) IsNot Nothing Then
                Me.ViewState(ViewStateVariable.PagerGridRecords) = value
            Else
                Me.ViewState.Add(ViewStateVariable.PagerGridRecords, value)
            End If
        End Set
    End Property

    Protected ReadOnly Property PageIndex() As Integer
        Get
            If Me.PageNumber > 1 Then
                Return Me.PageNumber - 1
            Else
                Return 0
            End If
        End Get
    End Property

    Protected Property ExcelFileName As String
        Get
            If Me.ViewState(ViewStateVariable.PagerExcelFileName) IsNot Nothing Then
                Return Convert.ToString(Me.ViewState(ViewStateVariable.PagerExcelFileName))
            Else
                Me.ViewState.Add(ViewStateVariable.PagerExcelFileName, DEFAULT_XLS_FILENAME)
                Return DEFAULT_XLS_FILENAME
            End If
        End Get
        Set(value As String)
            If Me.ViewState(ViewStateVariable.PagerExcelFileName) IsNot Nothing Then
                Me.ViewState(ViewStateVariable.PagerExcelFileName) = value
            Else
                Me.ViewState.Add(ViewStateVariable.PagerExcelFileName, value)
            End If
        End Set
    End Property

    Protected Property WorkSheetName As String
        Get
            If Me.ViewState(ViewStateVariable.PagerWorksheetName) IsNot Nothing Then
                Return Convert.ToString(Me.ViewState(ViewStateVariable.PagerWorksheetName))
            Else
                Me.ViewState.Add(ViewStateVariable.PagerWorksheetName, DEFAULT_WORKSHEET_NAME)
                Return DEFAULT_WORKSHEET_NAME
            End If
        End Get
        Set(value As String)
            If value.Length < 1 Or value.Length > 31 Then
                If Me.ViewState(ViewStateVariable.PagerWorksheetName) IsNot Nothing Then
                    Me.ViewState(ViewStateVariable.PagerWorksheetName) = DEFAULT_WORKSHEET_NAME
                Else
                    Me.ViewState.Add(ViewStateVariable.PagerWorksheetName, DEFAULT_WORKSHEET_NAME)
                End If
            Else
                If Me.ViewState(ViewStateVariable.PagerWorksheetName) IsNot Nothing Then
                    Me.ViewState(ViewStateVariable.PagerWorksheetName) = value
                Else
                    Me.ViewState.Add(ViewStateVariable.PagerWorksheetName, value)
                End If
            End If
        End Set
    End Property

    Protected Property ExcelDataExportMode As DataExportMode
        Get
            If Me.ViewState(ViewStateVariable.PagerExcelDataExportMode) IsNot Nothing Then
                Return CType(Convert.ToInt32(Me.ViewState(ViewStateVariable.PagerExcelDataExportMode)), DataExportMode)
            Else
                Me.ViewState.Add(ViewStateVariable.PagerExcelDataExportMode, DataExportMode.AllDataInDataSource)
                Return DataExportMode.AllDataInDataSource
            End If
        End Get
        Set(value As DataExportMode)
            If Me.ViewState(ViewStateVariable.PagerExcelDataExportMode) IsNot Nothing Then
                Me.ViewState(ViewStateVariable.PagerExcelDataExportMode) = value
            Else
                Me.ViewState.Add(ViewStateVariable.PagerExcelDataExportMode, value)
            End If
        End Set
    End Property

    Protected Property SelectedView() As Integer
        Get
            If Me.ViewState(ViewStateVariable.PagerSelectedViewIndex) IsNot Nothing Then
                Return Convert.ToInt32(Me.ViewState(ViewStateVariable.PagerSelectedViewIndex))
            Else
                Me.ViewState.Add(ViewStateVariable.PagerSelectedViewIndex, 0)
                Return 0
            End If
        End Get
        Set(value As Integer)
            If Me.ViewState(ViewStateVariable.PagerSelectedViewIndex) IsNot Nothing Then
                Me.ViewState(ViewStateVariable.PagerSelectedViewIndex) = value
            Else
                Me.ViewState.Add(ViewStateVariable.PagerSelectedViewIndex, value)
            End If
        End Set
    End Property

    Protected Property SelectedViewName As String
        Get
            If Me.ViewState(ViewStateVariable.PagerSelectedViewName) IsNot Nothing Then
                Return Convert.ToString(Me.ViewState(ViewStateVariable.PagerSelectedViewName))
            Else
                Me.ViewState.Add(ViewStateVariable.PagerSelectedViewName, DEFAULT_VIEW_NAME)
                Return DEFAULT_VIEW_NAME
            End If
        End Get
        Set(value As String)
            If Me.ViewState(ViewStateVariable.PagerSelectedViewName) IsNot Nothing Then
                Me.ViewState(ViewStateVariable.PagerSelectedViewName) = value
            Else
                Me.ViewState.Add(ViewStateVariable.PagerSelectedViewName, value)
            End If
        End Set
    End Property

    Protected ReadOnly Property ViewIndex() As Integer
        Get
            If Me.SelectedView > 1 Then
                Return Me.SelectedView
            Else
                Return 0
            End If
        End Get
    End Property

    Protected Property GridType() As GridType
        Get
            If Me.ViewState(ViewStateVariable.PagerGridType) IsNot Nothing Then
                Return CType(Convert.ToInt32(Me.ViewState(ViewStateVariable.PagerGridType)), GridType)
            Else
                Me.ViewState.Add(ViewStateVariable.PagerGridType, GridType.NotSet)
                Return GridType.NotSet
            End If
        End Get
        Set(value As GridType)
            If Me.ViewState(ViewStateVariable.PagerGridType) IsNot Nothing Then
                Me.ViewState(ViewStateVariable.PagerGridType) = value
            Else
                Me.ViewState.Add(ViewStateVariable.PagerGridType, value)
            End If
        End Set
    End Property

    <DefaultValue(True)>
    Protected Property ShowViews() As Boolean
        Get
            Return Me._showViews
        End Get
        Set(value As Boolean)
            Me._showViews = value
        End Set
    End Property

    <DefaultValue(True)>
    Protected Property ShowPager() As Boolean
        Get
            Return Me._showPager
        End Get
        Set(value As Boolean)
            Me._showPager = value
        End Set
    End Property

    <DefaultValue(True)>
    Protected Property ShowExcelExport() As Boolean
        Get
            Return Me._showExcelExport
        End Get
        Set(value As Boolean)
            Me._showExcelExport = value
        End Set
    End Property
#End Region

#Region " Page Events "
    Protected Sub Page_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        Dim scriptManager As ScriptManager = scriptManager.GetCurrent(Page)
        'scriptManager.RegisterPostBackControl(btnExportToExcel)
        scriptManager.RegisterPostBackControl(btnPageFirst)
        scriptManager.RegisterPostBackControl(btnPageNext)
        scriptManager.RegisterPostBackControl(btnPagePrev)
        scriptManager.RegisterPostBackControl(btnPageLast)
        'scriptManager.RegisterPostBackControl(ddlViews)
        'scriptManager.RegisterPostBackControl(ddlRows)
        'scriptManager.RegisterPostBackControl(ddlPageNumbers)
    End Sub

    Protected Sub Page_PreRender(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.PreRender
        Me.FillPageNumbersList()
        Me.FillPageSizeList()
        Me.FillViewsList()

        Me.FormatPageNumbersList()
        Me.FormatPageSizeList()
        Me.FormatArrowControls()
        Me.FormatViewsList()

        Initialize_Layout()

        If Not (Page.ClientScript.IsStartupScriptRegistered(Me.GetType(), Me.GetType().ToString)) Then
            Dim scriptBlock As String = "var g_GridPager_ExportButtonClicked = false;"
            Page.ClientScript.RegisterStartupScript(Me.GetType(), Me.GetType().ToString, scriptBlock, True)
        End If

        'Me.btnExcelExport.ClientSideEvents.Click = "g_GridPager_ExportButtonClicked = true;"
    End Sub
#End Region

#Region " Events "
    Private Sub RaiseExcelExportPageBeginEvent()
        RaiseEvent ExportedToExcel(Me, New GridPagerExportedToExcelEventArgs() With { _
            .ExcelFileName = Me.ExcelFileName, _
            .WorkSheetName = Me.WorkSheetName, _
            .ExcelDataExportMode = Me.ExcelDataExportMode, _
            .ExportToExcelFlag = True})
    End Sub

    Private Sub RaiseExcelExportPageCompletedEvent()
        RaiseEvent ExportedToExcel(Me, New GridPagerExportedToExcelEventArgs() With { _
            .ExcelFileName = "", _
            .WorkSheetName = "", _
            .ExcelDataExportMode = Me.ExcelDataExportMode, _
            .ExportToExcelFlag = False})

    End Sub

    Protected Sub changePage(sender As Object, e As EventArgs)
        If TypeOf sender Is DropDownList Then
            Me.PageNumber = Convert.ToInt32(DirectCast(sender, DropDownList).SelectedValue)
        ElseIf TypeOf sender Is LinkButton Then
            Select Case DirectCast(sender, LinkButton).CommandArgument
                Case "first"
                    Me.PageNumber = 1
                    Exit Select
                Case "prev"
                    Me.PageNumber -= 1
                    Exit Select
                Case "next"
                    Me.PageNumber += 1
                    Exit Select
                Case "last"
                    Me.PageNumber = Me.TotalNumberOfPages
                    Exit Select
                Case Else
                    Exit Select
            End Select
        Else
            Throw New NotImplementedException("changePage is not implemented for " & sender.GetType().Name)
        End If
        Me.RaiseChangePageEvent(Me.PageNumber)
    End Sub

    Private Sub RaiseChangePageEvent(pageNumber As Integer)
        RaiseEvent PageChanged(Me, New GridPagerSettingsChangedEventArgs() With { _
            .PageNumber = Me.PageNumber, _
            .TotalNumberOfPages = Me.TotalNumberOfPages})
    End Sub

    Protected Sub OnPageSizeChanged(sender As Object, e As EventArgs)
        Dim nNewPageSize As Integer = CInt(Me.ddlRows.SelectedValue)
        Me.NumberOfRows = nNewPageSize
        Me.PageSize = CInt(Me.ddlRows.SelectedValue)

        If Me.TotalNumberOfPages < Me.PageNumber Then
            Me.PageNumber = 1
            Me.RaiseChangePageEvent(1)
        End If

        Dim pageSizeChangedArgs As New GridPagerPageSizeChangedEventArgs(nNewPageSize)
        RaiseEvent NumberOfRowsChanged(Me, pageSizeChangedArgs)
    End Sub

    Protected Sub changeView(sender As Object, e As EventArgs)
        Me.SelectedView = DirectCast(sender, DropDownList).SelectedIndex
        Me.SelectedViewName = DirectCast(sender, DropDownList).SelectedValue
        Me.RaiseChangeIndexEvent(Me.SelectedView, Me.SelectedViewName)
    End Sub

    Private Sub RaiseChangeIndexEvent(selectedIndex As Integer, selectedViewName As String)
        RaiseEvent ViewChanged(Me, New GridPagerViewChangedEventArgs() With { _
            .SelectedViewIndex = selectedIndex, _
            .SelectedViewName = selectedViewName})
    End Sub

    Protected Sub btnFirst_Click(sender As Object, e As System.Web.UI.ImageClickEventArgs) Handles btnPageFirst.Click
        Me.PageNumber = 1
        Me.RaiseChangePageEvent(Me.PageNumber)
    End Sub

    Protected Sub btnPrevious_Click(sender As Object, e As System.Web.UI.ImageClickEventArgs) Handles btnPagePrev.Click
        Me.PageNumber -= 1
        Me.RaiseChangePageEvent(Me.PageNumber)
    End Sub

    Protected Sub btnNext_Click(sender As Object, e As System.Web.UI.ImageClickEventArgs) Handles btnPageNext.Click
        Me.PageNumber += 1
        Me.RaiseChangePageEvent(Me.PageNumber)
    End Sub

    Protected Sub btnLast_Click(sender As Object, e As System.Web.UI.ImageClickEventArgs) Handles btnPageLast.Click
        Me.PageNumber = Me.TotalNumberOfPages
        Me.RaiseChangePageEvent(Me.PageNumber)
    End Sub

    Protected Sub btnExportToExcel_Click(sender As Object, e As System.Web.UI.ImageClickEventArgs) Handles btnExportToExcel.Click
        Me.RaiseExcelExportPageBeginEvent()
    End Sub
#End Region

#Region " Methods "
    Public Sub SetContext(totalRecordCount As Integer, initialPageSize As Integer, excelFileName As String, worksheetName As String, _
                          excelDataExportMode As DataExportMode, gridRecords As Integer, selectedViewIndex As Integer, _
                          selectedViewName As String, gridType As GridType, _
                          Optional bshowViews As Boolean = True, Optional bShowPager As Boolean = True, _
                          Optional bshowExcelExport As Boolean = True)

        Me.TotalRecordCount = totalRecordCount
        Me.PageSize = initialPageSize
        Me.ExcelFileName = excelFileName
        Me.WorkSheetName = worksheetName
        Me.ExcelDataExportMode = excelDataExportMode
        Me.NumberOfRows = gridRecords
        Me.SelectedView = selectedViewIndex
        Me.SelectedViewName = selectedViewName
        Me.GridType = gridType
        Me.ShowViews = bshowViews
        Me.ShowPager = bShowPager
        Me.ShowExcelExport = bshowExcelExport

    End Sub

    Public Sub SetPageNumber(pageNumber As Integer)
        Me.PageNumber = pageNumber
    End Sub

    Private Sub Initialize_Layout()

        'If Not Me.ShowViews Then
        '    Me.lblViews.Visible = False
        '    Me.ddlViews.Visible = False
        'End If

        'If Not Me.ShowPager Then
        '    Me.divPager.Visible = False
        'Else
        '    Me.divPager.Visible = True
        'End If

        'If Not Me.ShowExcelExport Then
        '    Me.btnExcelExport.Visible = False
        'Else
        '    Me.btnExcelExport.Visible = True
        'End If

    End Sub

    Private Sub FillPageNumbersList()
        Me.ddlPageNumbers.Items.Clear()
        If Me.TotalNumberOfPages = 0 Then
            Me.ddlPageNumbers.Items.Add(New ListItem("1", "1"))
        Else
            For i As Integer = 1 To Me.TotalNumberOfPages
                Me.ddlPageNumbers.Items.Add(New ListItem(i.ToString(), i.ToString()))
            Next
        End If
    End Sub

    Private Sub FormatPageNumbersList()
        If Me.ddlPageNumbers.Items.Count >= Me.PageNumber Then
            Me.ddlPageNumbers.SelectedIndex = Me.PageIndex
        Else
            Me.ddlPageNumbers.SelectedIndex = 0
        End If
    End Sub

    Private Sub FillPageSizeList()
        Me.ddlRows.Items.Clear()
        Dim li As ListItem = Nothing
        Me.ddlRows.Items.Add(New ListItem(CStr(5), CStr(5)))
        Me.ddlRows.Items.Add(New ListItem(CStr(10), CStr(10)))
        Me.ddlRows.Items.Add(New ListItem(CStr(20), CStr(20)))
        Me.ddlRows.Items.Add(New ListItem(CStr(30), CStr(30)))
        Me.ddlRows.Items.Add(New ListItem(CStr(40), CStr(40)))
        Me.ddlRows.Items.Add(New ListItem(CStr(50), CStr(50)))
        Me.ddlRows.Items.Add(New ListItem(CStr(100), CStr(100)))
    End Sub

    Private Sub FormatPageSizeList()
        Me.ddlRows.SelectedValue = CStr(Me.PageSize)
    End Sub

    Private Sub FormatArrowControls()
        Dim pgNo As Integer = Me.PageNumber
        Me.btnPageFirst.Enabled = (pgNo <> 1)
        With Me.btnPageFirst
            If .Enabled Then
                .ImageUrl = IMAGE_DIRECTORY & GridPagerPropertyDefaults.DefaultFirstImage
            Else
                .ImageUrl = IMAGE_DIRECTORY & GridPagerPropertyDefaults.DefaultFirstDisabledImage
            End If
            .ImageAlign = ImageAlign.AbsMiddle
            .Width = Unit.Pixel(16)
            .Height = Unit.Pixel(16)
            .ToolTip = GridPagerPropertyDefaults.DefaultFirstToolTip
        End With

        Me.btnPagePrev.Enabled = (pgNo <> 1)
        With Me.btnPagePrev
            If .Enabled Then
                .ImageUrl = IMAGE_DIRECTORY & GridPagerPropertyDefaults.DefaultPreviousImage
            Else
                .ImageUrl = IMAGE_DIRECTORY & GridPagerPropertyDefaults.DefaultPreviousDisabledImage
            End If
            .ImageAlign = ImageAlign.AbsMiddle
            .Width = Unit.Pixel(16)
            .Height = Unit.Pixel(16)
            .ToolTip = GridPagerPropertyDefaults.DefaultPreviousToolTip
        End With

        Me.btnPageNext.Enabled = (pgNo <> Me.TotalNumberOfPages)
        With Me.btnPageNext
            If .Enabled Then
                .ImageUrl = IMAGE_DIRECTORY & GridPagerPropertyDefaults.DefaultNextImage
            Else
                .ImageUrl = IMAGE_DIRECTORY & GridPagerPropertyDefaults.DefaultNextDisabledImage
            End If
            .ImageAlign = ImageAlign.AbsMiddle
            .Width = Unit.Pixel(16)
            .Height = Unit.Pixel(16)
            .ToolTip = GridPagerPropertyDefaults.DefaultNextToolTip
        End With

        Me.btnPageLast.Enabled = (pgNo <> Me.TotalNumberOfPages)
        With Me.btnPageLast
            If .Enabled Then
                .ImageUrl = IMAGE_DIRECTORY & GridPagerPropertyDefaults.DefaultLastImage
            Else
                .ImageUrl = IMAGE_DIRECTORY & GridPagerPropertyDefaults.DefaultLastDisabledImage
            End If
            .ImageAlign = ImageAlign.AbsMiddle
            .Width = Unit.Pixel(16)
            .Height = Unit.Pixel(16)
            .ToolTip = GridPagerPropertyDefaults.DefaultLastToolTip
        End With

        With Me.btnExportToExcel
            .Enabled = True
            .ImageAlign = ImageAlign.AbsMiddle
            .ImageUrl = IMAGE_DIRECTORY & GridPagerPropertyDefaults.DefaultExportToExcelImage
            .Width = Unit.Pixel(16)
            .Height = Unit.Pixel(16)
            .ToolTip = GridPagerPropertyDefaults.DefaultExportToExcelToolTip
        End With

    End Sub

    Public Sub ExportGridToExcel(oGrid As WebHierarchicalDataGrid)
        Me.webExcelExporter.DataExportMode = Me.ExcelDataExportMode
        Me.webExcelExporter.DownloadName = HttpUtility.UrlEncode(ExcelFileName).Replace("+", "%20")
        Me.webExcelExporter.ExportMode = ExportMode.Download

        Dim excelFormat As Excel.WorkbookFormat = Excel.WorkbookFormat.Excel97To2003
        Dim wBook As New Excel.Workbook(excelFormat)
        wBook.Worksheets.Clear()
        Dim wSheet As Excel.Worksheet = wBook.Worksheets.Add(Me.WorkSheetName)
        Try
            Me.webExcelExporter.Export(oGrid, wSheet)
        Catch ex As ArgumentException
            Throw New ApplicationException(EXCEL_MAX_FIELD_LENGTH_ERROR)
        Finally
            RaiseExcelExportPageCompletedEvent()
        End Try
    End Sub

    Private Sub FillViewsList()
        Me.ddlViews.Items.Clear()

        Dim aryEnums As Array = [Enum].GetValues(GetType(enumViews))
        For i As Integer = 0 To aryEnums.GetLength(0) - 1
            Select Case aryEnums(i).ToString
                Case enumViews.DEFAULT_LIST.ToString
                    Me.ddlViews.Items.Insert(i, ViewConstants.DEFAULT_LIST)
                Case enumViews.BU_NATURE_OF_RECORD.ToString
                    Me.ddlViews.Items.Insert(i, ViewConstants.BU_NATURE_OF_RECORD)
                Case enumViews.DIRECTORATE_PROJECTS.ToString
                    Me.ddlViews.Items.Insert(i, ViewConstants.DIRECTORATE_PROJECTS)
                Case enumViews.MY_RECORDS.ToString
                    Me.ddlViews.Items.Insert(i, ViewConstants.MY_RECORDS)
                Case enumViews.PROJECTS.ToString
                    Me.ddlViews.Items.Insert(i, ViewConstants.PROJECTS)
                Case enumViews.RECORDS_BY_BUS_PROCESS.ToString
                    Me.ddlViews.Items.Insert(i, ViewConstants.RECORDS_BY_BUS_PROCESS)
                Case enumViews.RECORDS_BY_CATEGORY.ToString
                    Me.ddlViews.Items.Insert(i, ViewConstants.RECORDS_BY_CATEGORY)
                Case enumViews.RECORDS_BY_NATURE.ToString
                    Me.ddlViews.Items.Insert(i, ViewConstants.RECORDS_BY_NATURE)
                Case enumViews.RECORDS_BY_TYPE.ToString
                    Me.ddlViews.Items.Insert(i, ViewConstants.RECORDS_BY_TYPE)
                Case enumViews.RED_RECORDS.ToString
                    Me.ddlViews.Items.Insert(i, ViewConstants.RED_RECORDS)
                Case Else
                    Me.ddlViews.Items.Insert(i, ViewConstants.DEFAULT_LIST)
            End Select
        Next

    End Sub

    Private Sub FormatViewsList()
        If Me.ddlViews.Items.Count = 0 Then
            Me.FillViewsList()
        End If

        If Me.SelectedView < 0 Then
            If Me.ddlViews.Items.Count = 1 Then
                Me.ddlViews.SelectedIndex = 0
            Else
                Me.ddlViews.SelectedIndex = 0
            End If
        ElseIf Me.ddlViews.Items.Count >= Me.SelectedView Then
            Me.ddlViews.SelectedIndex = Me.SelectedView
        Else
            Me.ddlViews.SelectedIndex = 1
        End If
    End Sub
#End Region
End Class